
package mobicomp.hearts.lobby;


import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Observable;
import java.util.Observer;

import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.JTextArea;
import javax.swing.ListSelectionModel;
import javax.swing.border.BevelBorder;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;


/**
 * This class is the view in the model-view-controller 
 * pattern consisting of LobbyData, LobbyFrame, and LobbyController
 * 
 * Just some ugly swing code and an action listener 
 * 
 * @author Andreas Wetzel <andi@spamato.net>
 */
public class LobbyFrame extends JFrame implements Observer, ActionListener, ListSelectionListener
{
    private static final String ACTION_LOOK = "look";
    private static final String ACTION_HOST = "host";
    private static final String ACTION_START = "start";

    private LobbyController controller;

    private JLabel nameLabel;
    private JLabel idLabel;

    private JButton lookButton;
    private JButton hostButton;
    private JButton startButton;

    private JList gameList;
    private JTextArea infoField;

    public LobbyFrame( LobbyController controller) {
        super( "Lobby");

        // TODO: remove
        setDefaultCloseOperation( JFrame.EXIT_ON_CLOSE);

        this.controller = controller;
        buildGui();

    }

    private void buildGui() {
        JPanel mainpanel, panel, subpanel;

        mainpanel = new JPanel();
        mainpanel.setLayout( new BorderLayout());

        // build info-panel
        panel = new JPanel();
        panel.setLayout( new FlowLayout( FlowLayout.LEADING, 0, 0));
        GridBagConstraints gbc = new GridBagConstraints();
        subpanel = new JPanel();
        subpanel.setLayout( new GridBagLayout());
        gbc.anchor = GridBagConstraints.WEST;
        gbc.insets = new Insets( 2, 2, 2, 2);
        gbc.gridx = 1;
        gbc.gridy = 1;
        subpanel.add( new JLabel( "Name: "), gbc);
        gbc.gridy = 2;
        subpanel.add( new JLabel( "ID: "), gbc);
        gbc.gridx = 2;
        gbc.gridy = 1;
        gbc.anchor = GridBagConstraints.CENTER;
        gbc.fill = GridBagConstraints.HORIZONTAL;
        nameLabel = new JLabel();
        idLabel = new JLabel();
        subpanel.add( nameLabel, gbc);
        gbc.gridy = 2;
        subpanel.add( idLabel, gbc);
        panel.add( subpanel);

        // add info panel
        mainpanel.add( panel, BorderLayout.NORTH);

        // build button panel
        panel = new JPanel();
        panel.setLayout( new BorderLayout());

        lookButton = new JButton();
        lookButton.setActionCommand( ACTION_LOOK);
        lookButton.addActionListener( this);
        hostButton = new JButton();
        hostButton.setActionCommand( ACTION_HOST);
        hostButton.addActionListener( this);
        startButton = new JButton();
        startButton.setActionCommand( ACTION_START);
        startButton.addActionListener( this);

        panel.add( lookButton, BorderLayout.NORTH);
        panel.add( hostButton, BorderLayout.CENTER);
        panel.add( startButton, BorderLayout.SOUTH);

        // add button panel
        mainpanel.add( panel, BorderLayout.SOUTH);

        // build game list panel
        panel = new JPanel();
        panel.setLayout( new BorderLayout());
        gameList = new JList();
        gameList.setVisibleRowCount( 8);
        gameList.setPreferredSize( new Dimension( 6, 136));
        gameList.setBorder( new BevelBorder( BevelBorder.LOWERED));
        gameList.setSelectionMode( ListSelectionModel.SINGLE_SELECTION);
        gameList.addListSelectionListener( this);

        panel.add( gameList, BorderLayout.CENTER);
        infoField = new JTextArea( 4, 4);
        infoField.setLineWrap( true);
        infoField.setWrapStyleWord( true);
        // TODO: remove
        infoField.setBorder( new BevelBorder( BevelBorder.LOWERED));
        panel.add( infoField, BorderLayout.SOUTH);

        // add game list panel
        mainpanel.add( panel, BorderLayout.CENTER);

        getContentPane().setLayout( new BorderLayout( 2, 2));
        getContentPane().add( mainpanel, BorderLayout.CENTER);

        pack();
    }

    /**
     * @see java.util.Observer#update(java.util.Observable, java.lang.Object)
     */
    public void update( Observable o, Object arg) {
        LobbyData data = (LobbyData) o;
        idLabel.setText( Integer.toString( data.player.getId()));
        nameLabel.setText( data.player.getName());

        boolean listenabled = (!data.isHostingGame) && (!data.hasJoined);
        gameList.setListData( data.openGames);
        gameList.setEnabled( listenabled);
        infoField.setText( (data.currentGame != null) ? data.currentGame.getInfo() : "");

        lookButton.setText( (data.isLookingForGames) ? "Stop" : "Look for open Games");
        lookButton.setEnabled( !data.isHostingGame);

        hostButton.setText( (data.isHostingGame) ? "Abort" : "Host a new Game");
        hostButton.setEnabled( !data.hasJoined);

        // TODO: replace 4 with constant
        boolean startenabled = (data.isHostingGame) ? (data.currentGame.getPlayers().size() == 4)
                : (data.currentGame != null);
        startButton.setText( (data.isHostingGame) ? "Start Game" : (!data.hasJoined) ? "Join Game" : "Leave Game");
        startButton.setEnabled( startenabled);

        pack();
    }

    /**
     * @see java.awt.event.ActionListener#actionPerformed(java.awt.event.ActionEvent)
     */
    public void actionPerformed( ActionEvent e) {
        String cmd = e.getActionCommand();

        if ( cmd.equals( ACTION_LOOK) ) {
            controller.toggleLookForGames();
        }
        else if ( cmd.equals( ACTION_HOST) ) {
            controller.toggleHostGame();
        }
        else if ( cmd.equals( ACTION_START) ) {
            controller.changeGameState();
        }
        else {
            System.err.println( "Action '" + cmd + "' not yet implemented!");
        }
    }

    /**
     * @see javax.swing.event.ListSelectionListener#valueChanged(javax.swing.event.ListSelectionEvent)
     */
    public void valueChanged( ListSelectionEvent e) {
        int i = gameList.getSelectedIndex();
        if ( i >= 0 ) {
            controller.selectOpenGame( i);
        }
    }
}